import json
import logging
from typing import cast

from overhave.transport.http import BaseHttpClient, BearerAuth
from overhave.transport.http.api_client.models import (
    ApiEmulationRunResponse,
    ApiFeatureResponse,
    ApiFeatureTypeResponse,
    ApiTagResponse,
    ApiTestRunResponse,
    ApiTestUserResponse,
)
from overhave.transport.http.api_client.settings import OverhaveApiClientSettings
from overhave.transport.http.base_client import HttpMethod

logger = logging.getLogger(__name__)


class OverhaveApiClient(BaseHttpClient[OverhaveApiClientSettings]):
    """Client for overhave api."""

    def __init__(self, settings: OverhaveApiClientSettings):
        super().__init__(settings=settings)

    def get_feature_tags_item(self, value: str) -> ApiTagResponse:
        logger.debug("Start get feature tags item with [value: %s]", value)
        response = self._make_request(
            method=HttpMethod.GET,
            url=self._settings.get_feature_tags_item_url,
            params={"value": value},
            auth=BearerAuth(self._settings.auth_token),
        )

        logger.debug("Get tags item successfully")

        return cast("ApiTagResponse", self._parse_or_raise(response, ApiTagResponse))

    def get_feature_tags_list(self, value: str) -> list[ApiTagResponse]:
        logger.debug("Start get feature tags list with [value: %s]", value)
        response = self._make_request(
            method=HttpMethod.GET,
            url=self._settings.get_feature_tags_list_url,
            params={"value": value},
            auth=BearerAuth(self._settings.auth_token),
        )
        logger.debug("Get tags list successfully")

        return [ApiTagResponse.model_validate(data) for data in response.json()]

    def get_feature_types(self) -> list[ApiFeatureTypeResponse]:
        logger.debug("Start get feature types list")
        response = self._make_request(
            method=HttpMethod.GET,
            url=self._settings.get_feature_types_list_url,
            auth=BearerAuth(self._settings.auth_token),
        )
        logger.debug("Get feature types successfully")

        return [ApiFeatureTypeResponse.model_validate(data) for data in response.json()]

    def get_features_by_tag_id(self, tag_id: int) -> list[ApiFeatureResponse]:
        logger.debug("Start get feature with [tag_id: %s]", tag_id)
        response = self._make_request(
            method=HttpMethod.GET,
            url=self._settings.get_feature_url,
            params={"tag_id": tag_id},
            auth=BearerAuth(self._settings.auth_token),
        )
        logger.debug("Get feature successfully")

        return [ApiFeatureResponse.model_validate(data) for data in response.json()]

    def get_features_by_tag_value(self, tag_value: str) -> list[ApiFeatureResponse]:
        logger.debug("Start get feature with [tag_value: %s]", tag_value)
        response = self._make_request(
            method=HttpMethod.GET,
            url=self._settings.get_feature_url,
            params={"tag_value": tag_value},
            auth=BearerAuth(self._settings.auth_token),
        )
        logger.debug("Get feature successfully")

        return [ApiFeatureResponse.model_validate(data) for data in response.json()]

    def get_test_run(self, test_run_id: int) -> ApiTestRunResponse:
        logger.debug("Start get test run with [test_run_id: %s]", test_run_id)
        response = self._make_request(
            method=HttpMethod.GET,
            url=self._settings.get_test_run_url,
            params={"test_run_id": test_run_id},
            auth=BearerAuth(self._settings.auth_token),
        )
        logger.debug("Get test run successfully")

        return cast("ApiTestRunResponse", self._parse_or_raise(response, ApiTestRunResponse))

    def create_test_run(self, tag_value: str) -> list[str]:
        logger.debug("Start create test run with [tag_value: %s]", tag_value)
        response = self._make_request(
            method=HttpMethod.POST,
            url=self._settings.get_test_run_create_url,
            params={"tag_value": tag_value},
            auth=BearerAuth(self._settings.auth_token),
        )
        logger.debug("Create test run successfully")

        return cast(list[str], response.json())

    def get_emulation_runs(self, test_user_id: int) -> list[ApiEmulationRunResponse]:
        logger.debug("Start get list of EmulationRun")
        response = self._make_request(
            method=HttpMethod.GET,
            url=self._settings.get_emulation_run_list_url,
            params={"test_user_id": test_user_id},
            auth=BearerAuth(self._settings.auth_token),
        )
        logger.debug("Get list of EmulationRun successfully")
        return [ApiEmulationRunResponse.model_validate(data) for data in response.json()]

    def get_test_user_by_user_id(self, user_id: int) -> ApiTestUserResponse:
        logger.debug("Start get test user by user_id: %s", user_id)
        response = self._make_request(
            method=HttpMethod.GET,
            url=self._settings.get_test_user_url,
            params={"user_id": user_id},
            auth=BearerAuth(self._settings.auth_token),
        )
        logger.debug("Get test user by user_id successfully")
        return cast("ApiTestUserResponse", self._parse_or_raise(response, ApiTestUserResponse))

    def get_test_user_by_user_key(self, user_key: str) -> ApiTestUserResponse:
        logger.debug("Start get test user by user_key: %s", user_key)
        response = self._make_request(
            method=HttpMethod.GET,
            url=self._settings.get_test_user_url,
            params={"user_key": user_key},
            auth=BearerAuth(self._settings.auth_token),
        )
        logger.debug("Get test user by user_key successfully")
        return ApiTestUserResponse.model_validate(response.json())

    def get_test_users(self, feature_type: str, allow_update: bool) -> list[ApiTestUserResponse]:
        logger.debug("Start get test users with feature_type: %s and allow_update: %s", feature_type, allow_update)
        response = self._make_request(
            method=HttpMethod.GET,
            url=self._settings.get_test_user_list_url,
            params={
                "feature_type": feature_type,
                "allow_update": allow_update,
            },
            auth=BearerAuth(self._settings.auth_token),
        )
        logger.debug("Get tests users successfully")
        return [ApiTestUserResponse.model_validate(data) for data in response.json()]

    def delete_test_user(self, user_id: int) -> None:
        logger.debug("Start delete user by user_id: %s", user_id)
        self._make_request(
            method=HttpMethod.DELETE,
            url=self._settings.get_test_user_id_url(user_id),
            auth=BearerAuth(self._settings.auth_token),
        )
        logger.debug("Delete test user successfully")

    def get_test_user_specification(self, user_id: int) -> dict[str, str | None]:
        logger.debug("Start get user specification by user_id: %s", user_id)
        response = self._make_request(
            method=HttpMethod.GET,
            url=self._settings.get_test_user_id_spec_url(user_id),
            auth=BearerAuth(self._settings.auth_token),
        )
        logger.debug("Get user specification successfully")
        return cast(dict[str, str | None], response.json())

    def update_test_user_specification(self, user_id: int, specification: dict[str, str | None]) -> None:
        logger.debug("Start update user specification by user_id: %s", user_id)
        self._make_request(
            method=HttpMethod.PUT,
            url=self._settings.get_test_user_id_spec_url(user_id),
            data=json.dumps(specification),
            auth=BearerAuth(self._settings.auth_token),
        )
        logger.debug("Update user specification successfully")



from flask import Flask
from pydantic import SecretStr


def get_flask_app(template_folder: str, secret_key: SecretStr) -> Flask:
    app = Flask("admin", template_folder=template_folder)
    app.secret_key = secret_key.get_secret_value()
    app.config.update({"FLASK_ADMIN_SWATCH": "united", "FLASK_ADMIN_FLUID_LAYOUT": True})
    return app



from pydantic_settings import SettingsConfigDict

from overhave.transport.http import BaseHttpClientSettings
from overhave.transport.http.gitlab_client.objects import TokenType


class OverhaveGitlabClientSettings(BaseHttpClientSettings):
    """Settings for :class:`GitlabHttpClient`."""

    model_config = SettingsConfigDict(env_prefix="OVERHAVE_GITLAB_")

    auth_token: str | None = None
    token_type: TokenType = TokenType.OAUTH



import logging
import re
import typing

import ldap3
from ldap3.core.exceptions import LDAPExceptionError
from pydantic import SecretStr

from overhave.transport.ldap.settings import OverhaveLdapClientSettings

logger = logging.getLogger(__name__)


class LdapUserSearchAttributes(typing.TypedDict):
    """LDAP user search attributes."""

    memberOf: list[str]  # noqa: N815


class LDAPAuthenticator:
    """Overhave LDAP client."""

    def __init__(self, settings: OverhaveLdapClientSettings) -> None:
        self._server = ldap3.Server(
            settings.url,
            use_ssl=True,
            connect_timeout=settings.timeout.seconds,
        )
        self._settings = settings

    def _get_connection(self, login: str, password: SecretStr) -> ldap3.Connection:
        return ldap3.Connection(
            self._server,
            user=f"{self._settings.domain}{login}",
            password=password.get_secret_value(),
            receive_timeout=self._settings.timeout.seconds,
            read_only=True,
            auto_referrals=False,
            auto_bind=True,
        )

    def _ask_ad_usergroups(self, connection: ldap3.Connection, login: str) -> list[str]:
        # Выполнение поискового запроса
        connection.search(
            search_base=self._settings.dn,
            search_filter=f"(sAMAccountName={login})",
            search_scope=ldap3.SUBTREE,
            attributes=["memberOf"],
            time_limit=self._settings.timeout.seconds,
        )
        if connection.result.get("result", 1) != 0:
            return []
        if connection.response is None:  # pragma: no cover
            # should not happen - result was 0
            raise ValueError

        members_of: list[LdapUserSearchAttributes] = [
            el["attributes"].get("memberOf", []) for el in connection.response if "attributes" in el
        ]

        if len(members_of) != 1:
            raise ValueError("Multiple or zero users with the same login")

        member_of = members_of[0]

        p = re.compile("CN=(.*?),", re.IGNORECASE)
        return [
            p.match(x).group(1)  # type: ignore[union-attr]
            for x in list(filter(lambda x: "OU=Security Groups" in x or "OU=Mail Groups" in x, member_of))
        ]

    def get_user_groups(self, login: str, password: SecretStr) -> list[str] | None:
        try:
            with self._get_connection(login, password) as conn:
                return self._ask_ad_usergroups(conn, login)
        except LDAPExceptionError:
            logger.debug("Failed LDAP authorization for user: %s", login)
        return None



import httpx
from pydantic_settings import SettingsConfigDict

from overhave.transport.http.base_client import BaseHttpClientSettings


class OverhaveStashClientSettings(BaseHttpClientSettings):
    """Settings for :class:`StashHttpClient`."""

    model_config = SettingsConfigDict(env_prefix="OVERHAVE_STASH_")

    pr_path: str = "rest/api/1.0/projects/{project_key}/repos/{repository_name}/pull-requests"
    auth_token: str

    def get_pr_url(self, project_key: str, repository_name: str) -> httpx.URL:
        return httpx.URL(f"{self.url}/{self.pr_path.format(project_key=project_key, repository_name=repository_name)}")



import enum
from datetime import datetime
from typing import Literal

import allure
from pydantic import BaseModel


class TokenRequestData(BaseModel):
    """Model for OAuth2 request data."""

    grant_type: Literal["password"] = "password"
    username: str
    password: str


class ApiTagResponse(BaseModel):
    """Model for Tag response data."""

    id: int
    value: str
    created_by: str


class ApiFeatureTypeResponse(BaseModel):
    """Model for Feature Type response data."""

    id: int
    name: str


class ApiFeatureResponse(BaseModel):
    """Model for Feature response data."""

    id: int
    created_at: datetime
    name: str
    author: str
    type_id: int
    last_edited_by: str
    last_edited_at: datetime
    task: list[str]
    file_path: str
    released: bool
    severity: allure.severity_level

    feature_type: ApiFeatureTypeResponse
    feature_tags: list[ApiTagResponse]


class ApiTestRunResponse(BaseModel):
    """Model for TestRun response data."""

    id: int
    created_at: datetime
    name: str
    executed_by: str
    start: datetime | None
    end: datetime | None
    status: str
    report_status: str
    report: str | None
    traceback: str | None
    scenario_id: int


class ApiEmulationStatus(enum.StrEnum):
    """Enum for Emulation Status."""

    CREATED = "CREATED"
    REQUESTED = "REQUESTED"
    READY = "READY"
    ERROR = "ERROR"


class ApiTestUserResponse(BaseModel):
    """Model for TestUser response data."""

    id: int
    created_at: datetime
    key: str
    name: str
    created_by: str
    specification: dict[str, str | None]
    feature_type_id: int
    feature_type: ApiFeatureTypeResponse
    allow_update: bool
    changed_at: datetime


class ApiEmulationResponse(BaseModel):
    """Model for Emulation response data."""

    id: int
    command: str
    test_user: ApiTestUserResponse


class ApiEmulationRunResponse(BaseModel):
    """Model for EmulationRun response data."""

    id: int
    emulation_id: int
    changed_at: datetime
    status: ApiEmulationStatus
    port: int | None
    initiated_by: str
    emulation: ApiEmulationResponse



import logging
from typing import cast

import flask
import werkzeug
from flask_admin import expose
from flask_admin.model.helpers import get_mdict_item_or_list
from flask_login import current_user
from wtforms import Form, ValidationError

from overhave import db
from overhave.admin.views.base import ModelViewConfigured
from overhave.factory import get_admin_factory, get_test_execution_factory
from overhave.pytest_plugin import get_proxy_manager
from overhave.transport import TestRunData, TestRunTask

logger = logging.getLogger(__name__)


class TestRunView(ModelViewConfigured):
    """View for :class:`TestRun` table."""

    __test__ = False

    list_template = "test_run_list.html"
    details_template = "test_run_detail.html"
    can_create = False
    can_edit = True
    column_searchable_list = (
        "name",
        "executed_by",
    )
    column_list = (
        "id",
        "name",
        "start",
        "end",
        "executed_by",
        "status",
    )
    column_details_list = (
        "id",
        "name",
        "start",
        "end",
        "executed_by",
        "status",
        "report_status",
        "report",
        "traceback",
    )
    column_filters = (
        "name",
        "start",
        "executed_by",
        "status",
    )
    column_descriptions = {
        "name": "Feature name",
        "executed_by": "Initiator of scenarios set test run",
        "status": "Test run result",
    }

    def on_model_change(self, form: Form, model: db.TestRun, is_created: bool) -> None:
        if not is_created and current_user.role != db.Role.admin:
            raise ValidationError("Only administrator could change test run data!")

    def on_model_delete(self, model: db.TestRun) -> None:
        if not (current_user.login == model.executed_by or current_user.role == db.Role.admin):
            raise ValidationError("Only test run initiator could delete test run result!")

    @staticmethod
    def _run_test(rendered: werkzeug.Response) -> werkzeug.Response:
        current_scenario_id = int(get_mdict_item_or_list(flask.request.args, "id"))
        factory = get_admin_factory()
        test_run_id = factory.test_run_storage.create_testrun(
            scenario_id=current_scenario_id, executed_by=current_user.login
        )

        if not factory.context.admin_settings.consumer_based:
            proxy_manager = get_proxy_manager()
            test_execution_factory = get_test_execution_factory()
            proxy_manager.clear_factory()
            proxy_manager.set_factory(test_execution_factory)
            factory.threadpool.apply_async(get_test_execution_factory().test_executor.execute_test, args=(test_run_id,))
        if factory.context.admin_settings.consumer_based and not factory.redis_producer.add_task(
            TestRunTask(data=TestRunData(test_run_id=test_run_id))
        ):
            flask.flash("Problems with Redis service! TestRunTask has not been sent.", category="error")
            return rendered
        logger.debug("Redirect to TestRun details view with test_run_id='%s'...", test_run_id)
        return flask.redirect(flask.url_for("testrun.details_view", id=test_run_id))

    @expose("/details/", methods=("GET", "POST"))
    def details_view(self) -> werkzeug.Response:
        rendered: werkzeug.Response = super().details_view()

        if flask.request.method == "POST":
            return self._run_test(rendered)

        return cast(werkzeug.Response, super().details_view())



import abc
import logging
import socket
from typing import List, cast

import orjson
import sqlalchemy as sa
from redis import Redis

from overhave import db
from overhave.entities.settings import OverhaveEmulationSettings
from overhave.storage import EmulationRunModel
from overhave.utils import get_current_time

logger = logging.getLogger(__name__)


class EmulationStorageError(Exception):
    """Base Exception for :class:`EmulationStorage`."""


class AllPortsAreBusyError(EmulationStorageError):
    """Exception for situation when all ports are busy."""


class IEmulationStorage(abc.ABC):
    """Abstract class for emulation runs storage."""

    @staticmethod
    @abc.abstractmethod
    def create_emulation_run(emulation_id: int, initiated_by: str) -> int:
        pass

    @abc.abstractmethod
    def get_requested_emulation_run(self, emulation_run_id: int) -> EmulationRunModel:
        pass

    @abc.abstractmethod
    def set_emulation_run_status(self, emulation_run_id: int, status: db.EmulationStatus) -> None:
        pass

    @abc.abstractmethod
    def set_error_emulation_run(self, emulation_run_id: int, traceback: str) -> None:
        pass

    @staticmethod
    @abc.abstractmethod
    def get_emulation_runs_by_test_user_id(test_user_id: int) -> list[EmulationRunModel]:
        pass

    @abc.abstractmethod
    def has_running_emulation_with_user(self, test_user_id: int) -> bool:
        pass


class EmulationStorage(IEmulationStorage):
    """Class for emulation runs storage."""

    def __init__(self, settings: OverhaveEmulationSettings, redis: Redis):
        self._redis = redis
        self._settings = settings
        self._redis.set(self._settings.redis_ports_key, orjson.dumps([]))
        self._emulation_ports_len = len(self._settings.emulation_ports)

    @staticmethod
    def create_emulation_run(emulation_id: int, initiated_by: str) -> int:
        with db.create_session() as session:
            emulation_run = db.EmulationRun(
                emulation_id=emulation_id, initiated_by=initiated_by, status=db.EmulationStatus.CREATED
            )
            session.add(emulation_run)
            session.flush()
            return emulation_run.id

    def _get_next_port(self) -> int:
        allocated_ports = self.get_allocated_ports()

        logger.debug("Allocated ports: %s", allocated_ports)
        not_allocated_ports = set(self._settings.emulation_ports).difference(allocated_ports)
        logger.debug("Not allocated ports: %s", not_allocated_ports)
        if not_allocated_ports:
            for port in not_allocated_ports:
                if self._is_port_in_use(port):
                    continue
                return port
            logger.debug("All not allocated ports are busy!")
        for port in allocated_ports:
            if self._is_port_in_use(port):
                continue
            return port
        raise AllPortsAreBusyError("All ports are busy - could not find free port!")

    def get_allocated_ports(self) -> List[int]:
        port_user_pairs = self.get_allocated_port_user_pairs()
        return [port for port, _ in port_user_pairs]

    def get_allocated_port_user_pairs(self) -> List[List[int]]:
        allocated_port_user_pairs = cast(bytes | None, self._redis.get(self._settings.redis_ports_key))
        logger.debug("allocated port user pairs: %s", allocated_port_user_pairs)
        if allocated_port_user_pairs is None:
            return []
        return cast(List[List[int]], orjson.loads(allocated_port_user_pairs))

    def allocate_port_for_user(self, port: int, test_user_id: int) -> None:
        new_allocated_ports = self.get_allocated_port_user_pairs()
        if [port, test_user_id] in new_allocated_ports:
            logger.debug("port %s for user %s already in redis: %s", port, test_user_id, new_allocated_ports)
            return
        new_allocated_ports.append([port, test_user_id])
        logger.debug("added port %s for user %s in redis: %s", port, test_user_id, new_allocated_ports)
        self._redis.set(self._settings.redis_ports_key, orjson.dumps(sorted(new_allocated_ports)))

    def _is_port_in_use(self, port: int) -> bool:
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            return s.connect_ex((self._settings.emulation_bind_ip, port)) == 0

    def get_requested_emulation_run(self, emulation_run_id: int) -> EmulationRunModel:
        with db.create_session() as session:
            emulation_run = session.query(db.EmulationRun).filter(db.EmulationRun.id == emulation_run_id).one()
            emulation_run.status = db.EmulationStatus.REQUESTED
            emulation_run.port = self._get_next_port()
            self.allocate_port_for_user(emulation_run.port, emulation_run.emulation.test_user_id)
            emulation_run.changed_at = get_current_time()
            return EmulationRunModel.model_validate(emulation_run)

    def set_emulation_run_status(self, emulation_run_id: int, status: db.EmulationStatus) -> None:
        with db.create_session() as session:
            session.execute(
                sa.update(db.EmulationRun)
                .where(db.EmulationRun.id == emulation_run_id)
                .values(status=status, changed_at=get_current_time())
            )

    def set_error_emulation_run(self, emulation_run_id: int, traceback: str) -> None:
        with db.create_session() as session:
            session.execute(
                sa.update(db.EmulationRun)
                .where(db.EmulationRun.id == emulation_run_id)
                .values(status=db.EmulationStatus.ERROR, changed_at=get_current_time(), traceback=traceback)
            )

    @staticmethod
    def get_emulation_runs_by_test_user_id(test_user_id: int) -> list[EmulationRunModel]:
        with db.create_session() as session:
            emulation_ids_query = (
                session.query(db.Emulation)
                .with_entities(db.Emulation.id)
                .filter(db.Emulation.test_user_id == test_user_id)
                .scalar_subquery()
            )
            emulation_runs = (
                session.query(db.EmulationRun).where(db.EmulationRun.emulation_id.in_(emulation_ids_query)).all()
            )
            return [EmulationRunModel.model_validate(x) for x in emulation_runs]

    def has_running_emulation_with_user(self, test_user_id: int) -> bool:
        port_user_pairs = self.get_allocated_port_user_pairs()

        for port, user in port_user_pairs:
            if user == test_user_id and self._is_port_in_use(port):
                return True
        return False



import httpx
from pydantic import StrictStr
from pydantic_settings import SettingsConfigDict

from overhave.base_settings import OVERHAVE_ENV_PREFIX
from overhave.transport.http.base_client import BaseHttpClientSettings


class OverhaveApiAuthenticatorSettings(BaseHttpClientSettings):
    """Settings for :class:`OverhaveApiAuthenticator`."""

    model_config = SettingsConfigDict(env_prefix=OVERHAVE_ENV_PREFIX + "API_AUTH_")

    auth_token_path: str = "token"

    @property
    def get_auth_token_url(self) -> httpx.URL:
        return httpx.URL(f"{self.url}/{self.auth_token_path}")


class OverhaveApiClientSettings(BaseHttpClientSettings):
    """Settings for :class:`OverhaveApiClient`."""

    model_config = SettingsConfigDict(env_prefix="OVERHAVE_API_CLIENT_")

    auth_token: str
    feature_path: StrictStr = StrictStr("feature/")
    feature_tags_item_path: StrictStr = StrictStr("feature/tags/item")
    feature_tags_list_path: StrictStr = StrictStr("feature/tags/list")
    feature_types_list_path: StrictStr = StrictStr("feature/types/list")

    test_run_path: StrictStr = StrictStr("test_run")
    test_run_create_path: StrictStr = StrictStr("test_run/create/")

    emulation_run_list_path: StrictStr = StrictStr("emulation/run/list")

    test_user_path: StrictStr = StrictStr("test_user/")
    test_user_list_path: StrictStr = StrictStr("test_user/list")

    @property
    def get_feature_url(self) -> httpx.URL:
        return httpx.URL(f"{self.url}/{self.feature_path}")

    @property
    def get_feature_tags_item_url(self) -> httpx.URL:
        return httpx.URL(f"{self.url}/{self.feature_tags_item_path}")

    @property
    def get_feature_tags_list_url(self) -> httpx.URL:
        return httpx.URL(f"{self.url}/{self.feature_tags_list_path}")

    @property
    def get_feature_types_list_url(self) -> httpx.URL:
        return httpx.URL(f"{self.url}/{self.feature_types_list_path}")

    @property
    def get_test_run_url(self) -> httpx.URL:
        return httpx.URL(f"{self.url}/{self.test_run_path}")

    @property
    def get_test_run_create_url(self) -> httpx.URL:
        return httpx.URL(f"{self.url}/{self.test_run_create_path}")

    @property
    def get_emulation_run_list_url(self) -> httpx.URL:
        return httpx.URL(f"{self.url}/{self.emulation_run_list_path}")

    @property
    def get_test_user_url(self) -> httpx.URL:
        return httpx.URL(f"{self.url}/{self.test_user_path}")

    @property
    def get_test_user_list_url(self) -> httpx.URL:
        return httpx.URL(f"{self.url}/{self.test_user_list_path}")

    def get_test_user_id_url(self, user_id: int) -> httpx.URL:
        return httpx.URL(f"{self.url}/test_user/{user_id}")

    def get_test_user_id_spec_url(self, user_id: int) -> httpx.URL:
        return httpx.URL(f"{self.url}/test_user/{user_id}/specification")



from datetime import timedelta

from pydantic_settings import BaseSettings, SettingsConfigDict


class OverhaveLdapClientSettings(BaseSettings):
    """Settings for Overhave LDAP client."""

    model_config = SettingsConfigDict(env_prefix="OVERHAVE_LDAP_")

    url: str  # for example: "ldap://mydomain.ru"
    domain: str  # for example: "domain\\"
    dn: str  # for example: "dc=example,dc=com"
    timeout: timedelta = timedelta(seconds=10)
    tls_enabled: bool = True
